
! ***********************************************************************
!                   CABEZON, GARCIA-SENZ & BRAVO (ApJS, 2004)
! ***********************************************************************
!
!     *** temp : Input temperature. 
!     *** tempout: Output temperature after calculation.
!
!     *** delta : Input time-step of the current model (s).
!
!     *** xin : Input array of mass fractions. 
!     *** xout: Output mass fractions.
!
!     *** ro : Input density (g/cm**3).
!
!     *** cv: Input heat capacity.
!
!     *** value1: Input (dp/dTemp)*(deltaro/ro**2)       [CGS]
!     *** value2: Input (dp/dTemp)*(deltaro/ro**2)*Temp  [CGS]
!
!     *** screen : Input. Turns the screening corrections to the rates on/off.
!     *** theta : Input. =1 for fully implicit method.
!
!     *** sumtot : Output total mass fraction (=1 if conserved)
!     *** nucenergy : Output total nuclear energy generation rate (erg/s)
!
! ***************
!     
!     *** a: array of mass number of all isotopes.
!     *** be: array of binding energy of all isotopes refered to 4He.
!     *** Isotopes order: (From 2 to 15)
!     (4He,12C,16O,20Ne,24Mg,28Si,32S,36Ar,40Ca,44Ti,48Cr,52Fe,56Ni,60Zn)
!     (Position 1 of array is not used)
!
!     *** eff: array of the direct reaction rates values.
!     *** l: array of the inverse reaction rates values.
!     *** deff, dl: derivatives of eff and l respect to the temperature.
!
!     *** Reactions order: (From 2 to 17)
!     (From 2 to 14) triple alpha, 12C(a,g)16O, 16O(a,g)20Ne, ...
!     (From 15 to 17) Heavy Ion: 12C(12C,a)20Ne, 12C(16O,a)24Mg, 16O(16O,a)28Si
!
! ***********************************************************************
! ***********************************************************************
  SUBROUTINE net14(temp,tempout,delta,xin,xout,ro,screen,cv,&
                 & value1,value2,sumtot,nucenergy,nP,theta)
      USE nuclear_module

      IMPLICIT NONE
      DOUBLE PRECISION,INTENT(in)::temp,delta,ro,cv,value1,value2,theta
      DOUBLE PRECISION,INTENT(out)::tempout,sumtot,nucenergy

      INTEGER,INTENT(in)::nP
      LOGICAL,INTENT(in)::screen

      DOUBLE PRECISION,INTENT(in),DIMENSION(niso)::xin
      DOUBLE PRECISION,INTENT(out),DIMENSION(niso)::xout
      
      DOUBLE PRECISION EF,max

      DOUBLE PRECISION,DIMENSION(iso)::vindep,norm
      DOUBLE PRECISION,DIMENSION(niso)::x

      if(nP.ne.niso) then
         write(*,*) 'nP (net14) and niso (nuclear_module) are not equal!'
         write(*,*) 'nP: ',nP,' niso: ',niso
         stop
      endif

! **********************
! *** Initialization ***
! **********************

      if(theta.eq.0.d0) then
         print *,'Wrong theta value!' 
         print *,'0 < Theta <= 1'
         stop
      endif

      do i=1,4
        q(i)=0.d0
        do j=1,8
           fit(i,j)=0.d0
        enddo
      enddo
      do i=1,4
         do j=1,24
            choose(i,j)=0.d0
         enddo
      enddo

      y(1)=0.d0
      do i=2,15
        y(i)=xin(i-1)/a(i)
      enddo

! ***********************************************************************
! *********************************************************************** 

 
! OOOOOOOOOOOOOOOOOOO START RUN OOOOOOOOOOOOOOOOOOO

! Calculation of the chemical potentials.

      if(screen)call chempot(temp,ro)

! Calculation of the rates.
! Till 16O(a,g)20Ne, including heavy ion reactions: calculated with WWFHZ78
!    using subroutines 'effect' and 'deffcalc'.
! From 20Ne(a,g)24Mg till the end of the network: calculated with RT00
!    using subroutines 'direct' and 'inverse'.

      call effect(temp)
      call deffcalc(temp)
      call direct(temp)
      call inverse(temp)
      
      do i=2,17
        eff(i)=eff(i)*ro
        deff(i)=deff(i)*ro
      enddo
      eff(2)=eff(2)*ro
      deff(2)=deff(2)*ro

! Rates corrections due to the chemical potentials.
! It only affects the direct reaction rates and their derivatives.

      if(screen)then
         do 9 i=2,17
            EF=dexp(deltamukbt(i))
            eff(i)=eff(i)*EF
            deff(i)=deff(i)*EF-2.d0*eff(i)*deltamukbt(i)/temp
 9       enddo
      endif

! Function and matrix calculation
! myfunc: Function evaluation that will become the indepent term.
! myfuncprima: Same as myfunc but evaluated with the rates derivatives instead 
!    of the rates themselves.
! phicalc: matrix of coeficients

      call myfunc()
      call myfuncprima()
      call phicalc(delta,temp,ro,cv)
 
! Calculates the temperature term of the energy equation.

      phi(1,1)=cv-value1*theta

      do j=1,15
         do i=2,15
            phi(i,j)=phi(i,j)*theta
         enddo
      enddo
! OOOOOOOOOOOOOOOOOOO SYSTEM RESOLUTION OOOOOOOOOOOOOOOOOOO
      
! Matrix normalization.
! norm = array with the normalization values for every equation of the system.
! vindep = array containing the independent terms of the equations.

      do i=1,15
         max=dabs(phi(i,1))
         do j=1,15
            if(dabs(phi(i,j)).gt.max) max=dabs(phi(i,j))
         enddo
         norm(i)=max
         if(max.eq.0.d0)then
            write(*,*) 'Zero row in the matrix...!!'
            stop
         endif
         do j=1,15
            phi(i,j)=phi(i,j)/max
         enddo
      enddo

      vindep(1)=value2/norm(1)
      
      do i=2,15
         vindep(i)=delta*f(i)/norm(i)
      enddo

! An additional column in the matrix is needed for subroutine 'eigen' to run correctly. 
! In this column is stored the array of the independent terms of the system.

      do i=1,15
         phi(i,16)=vindep(i)
      enddo

      call eigen(15,vindep,vindep)

! Abundances and temperature update.

      do i=2,15
         y(i)=y(i)+vindep(i)
!         if(y(i).lt.0.d0) then
!            write(*,*) 'Warning! Negative value of y number:',i
!            write(*,*) 'y(',i,')=',y(i)
!         endif
         if(y(i).le.ymin)y(i)=0.d0
      enddo
      tempout=temp+vindep(1)

! OOOOOOOOOOOOOOOOOOO SYSTEM RESOLUTION END OOOOOOOOOOOOOOOOOOO

      sumtot=0.d0
      do i=2,15
         xout(i-1)=y(i)*a(i)
         sumtot=sumtot+xout(i-1)
      enddo

      nucenergy=0.d0
      do i=2,15
         nucenergy=nucenergy+vindep(i)*BE(i)
      enddo
      return
    end subroutine net14

!*******************************************************************
! *** SUBROUTINE direct.
! *** Calculates the direct nuclear reaction rates using RT00
! *** From 20Ne(a,g)24Mg to 56Ni(a,g)60Zn

      SUBROUTINE direct(temp)
        
      USE nuclear_module

      IMPLICIT NONE

      DOUBLE PRECISION,INTENT(in)::temp
      
      DOUBLE PRECISION t9,t913,t923,t953,t9i,t9i2,t9i13,t9i23,t9i43,lt9

      t9=temp/1.d9
      t913=t9**(third)
      t923=t913*t913
      t953=t9**(5.d0*third)
      t9i=1.d0/t9
      t9i2=t9i*t9i
      t9i13=1.d0/t913
      t9i23=t9i13*t9i13
      t9i43=t9i23*t9i23
      lt9=dlog(t9)

      do i=5,14
         coef(i)=fit(i,2)*t9i+fit(i,3)*t9i13+fit(i,4)*t913+fit(i,5)*t9&
              &+fit(i,6)*t953+fit(i,7)*lt9
         dcoef(i)=(-fit(i,2)*t9i2+(-fit(i,3)*t9i43+fit(i,4)*t9i23+5.d0&
              &*fit(i,6)*t923)*(third)+fit(i,5)+fit(i,7)*t9i)*1.d-9
         eff(i)=dexp(fit(i,1)+coef(i))
         deff(i)=eff(i)*dcoef(i)
      enddo
      return
      end subroutine direct

!*******************************************************************
! *** SUBROUTINE inverse.
! *** Calculates the inverse nuclear reaction rates using RT00
! *** From 20Ne(a,g)24Mg to 56Ni(a,g)60Zn

      SUBROUTINE inverse(temp)

      USE nuclear_module

      IMPLICIT NONE

      DOUBLE PRECISION,INTENT(in)::temp

      DOUBLE PRECISION t9,t9i,val1,val2,val3,val4,part
           
      t9=temp/1.d9
      t9i=1.d0/t9

      if(t9.ge.0.01d0.and.t9.lt.0.15d0) then
        k=1
      else if(t9.ge.0.15d0.and.t9.lt.0.2d0) then
        k=2
      else if(t9.ge.0.2d0.and.t9.lt.0.3d0) then
        k=3
      else if(t9.ge.0.3d0.and.t9.lt.0.4d0) then
        k=4
      else if(t9.ge.0.4d0.and.t9.lt.0.5d0) then
        k=5
      else if(t9.ge.0.5d0.and.t9.lt.0.6d0) then
        k=6
      else if(t9.ge.0.6d0.and.t9.lt.0.7d0) then
        k=7
      else if(t9.ge.0.7d0.and.t9.lt.0.8d0) then
         k=8
      else if(t9.ge.0.8d0.and.t9.lt.0.9d0) then
        k=9
      else if(t9.ge..9d0.and.t9.lt.1.d0) then
        k=10
      else if(t9.ge.1.d0.and.t9.lt.1.5d0) then
        k=11
      else if(t9.ge.1.5d0.and.t9.lt.2.d0) then
        k=12
      else if(t9.ge.2.d0.and.t9.lt.2.5d0) then
        k=13
      else if(t9.ge.2.5d0.and.t9.lt.3.d0) then
        k=14
      else if(t9.ge.3.d0.and.t9.lt.3.5d0) then
        k=15
      else if(t9.ge.3.5d0.and.t9.lt.4.d0) then
        k=16
      else if(t9.ge.4.d0.and.t9.lt.4.5d0) then
        k=17
      else if(t9.ge.4.5d0.and.t9.lt.5.d0) then
        k=18
      else if(t9.ge.5.d0.and.t9.lt.6.d0) then
        k=19
      else if(t9.ge.6.d0.and.t9.lt.7.d0) then
        k=20
      else if(t9.ge.7.d0.and.t9.lt.8.d0) then
        k=21
      else if(t9.ge.8.d0.and.t9.lt.9.d0) then
        k=22
      else if(t9.ge.9.d0.and.t9.lt.10.d0) then
        k=23
      else if(t9.ge.10.d0) then
        k=24
      endif

      val1=11.6045d0*t9i
      val2=1.5d0*dlog(t9)
      val3=val1*t9i*1.d-9
      val4=1.5d-9*t9i
      do i=5,14
        part=choose(i,k)/choose(i+1,k)
        l(i)=part*dexp(fit(i,8)+coef(i)-val1*q(i)+val2)
        dl(i)=l(i)*(dcoef(i)+val3*q(i)+val4)
      enddo
      return
      end SUBROUTINE inverse
      
!*******************************************************************

      SUBROUTINE myfunc()

      USE nuclear_module

      IMPLICIT NONE

      do i=1,15
        f(i)=0.d0
      enddo
      do i=4,14
         f(i)=l(i)*y(i+1)-l(i-1)*y(i)-eff(i)*y(i)*y(2)+eff(i-1)*y(i-1)*y(2)
      enddo
      do i=3,14
         f(2)=f(2)+l(i-1)*y(i)-eff(i)*y(i)*y(2)
      enddo
      f(2)=f(2)+2.d0*l(2)*y(3)+.5d0*eff(15)*y(3)*y(3)+eff(16)*y(3)*y(4)&
           &+.5d0*eff(17)*y(4)*y(4)-.5d0*eff(2)*y(2)*y(2)*y(2)+l(14)*y(15)
      f(3)=l(3)*y(4)-l(2)*y(3)-y(3)*(eff(3)*y(2)+eff(15)*y(3)&
           &+eff(16)*y(4))+eff(2)*y(2)*y(2)*y(2)/6.d0
      f(4)=f(4)-y(4)*(eff(16)*y(3)+eff(17)*y(4))
      f(5)=f(5)+.5d0*eff(15)*y(3)**2
      f(6)=f(6)+eff(16)*y(3)*y(4)
      f(7)=f(7)+.5d0*eff(17)*y(4)**2
      f(15)=-l(14)*y(15)+eff(14)*y(14)*y(2)
      RETURN
      END SUBROUTINE myfunc

!*******************************************************************

      SUBROUTINE myfuncprima()

      USE nuclear_module

      IMPLICIT NONE

      do i=1,15
        fprima(i)=0.d0
      enddo
      do i=4,14
         fprima(i)=dl(i)*y(i+1)-dl(i-1)*y(i)-deff(i)*y(i)*y(2)&
              &+deff(i-1)*y(i-1)*y(2)
      enddo
      do i=3,14
         fprima(2)=fprima(2)+dl(i-1)*y(i)-deff(i)*y(i)*y(2)
      enddo
      fprima(2)=fprima(2)+2.*dl(2)*y(3)+.5d0*deff(15)*y(3)*y(3)&
           &+deff(16)*y(3)*y(4)+.5d0*deff(17)*y(4)*y(4)&
           &-.5d0*deff(2)*y(2)*y(2)*y(2)+dl(14)*y(15)
      fprima(3)=dl(3)*y(4)-dl(2)*y(3)-y(3)*(deff(3)*y(2)+deff(15)*y(3)&
           &+deff(16)*y(4))+deff(2)*y(2)*y(2)*y(2)/6.d0
      fprima(4)=fprima(4)-y(4)*(deff(16)*y(3)+deff(17)*y(4))
      fprima(5)=fprima(5)+.5d0*deff(15)*y(3)**2
      fprima(6)=fprima(6)+deff(16)*y(3)*y(4)
      fprima(7)=fprima(7)+.5d0*deff(17)*y(4)**2
      fprima(15)=-dl(14)*y(15)+deff(14)*y(14)*y(2)

      RETURN
      END SUBROUTINE myfuncprima

!*******************************************************************

      SUBROUTINE phicalc(delta,temp,ro,cv)

      USE nuclear_module

      IMPLICIT NONE

      DOUBLE PRECISION,INTENT(in)::delta,temp,ro,cv
      DOUBLE PRECISION kbt,nakbt,ne,ae,gam,gamma
      DOUBLE PRECISION funcion,ggt1,glt1

      kbt=1.380658d-16*temp
      nakbt=na*kbt

      do j=1,16
        do i=1,15
          phi(i,j)=0.d0
        enddo
      enddo

      do i=4,14
         phi(i,i-1)=-delta*eff(i-1)*y(2)
         phi(i,i)=1.d0-delta*(-l(i-1)-eff(i)*y(2))
         phi(i,i+1)=-delta*l(i)
         phi(i,2)=-delta*(-eff(i)*y(i)+eff(i-1)*y(i-1))
      enddo
      phi(4,3)=phi(4,3)+delta*eff(16)*y(4)
      phi(4,4)=phi(4,4)+delta*(eff(16)*y(3)+2.d0*eff(17)*y(4))
      phi(5,3)=-delta*eff(15)*y(3)
      phi(6,3)=-delta*eff(16)*y(4)
      phi(6,4)=-delta*eff(16)*y(3)
      phi(7,4)=-delta*eff(17)*y(4)
      phi(15,2)=-delta*eff(14)*y(14)
      phi(15,14)=-delta*eff(14)*y(2)
      phi(15,15)=1.d0+delta*l(14)
      do i=3,14
         phi(2,i)=-delta*(l(i-1)-eff(i)*y(2))
         phi(2,2)=phi(2,2)-eff(i)*y(i)
      enddo
      phi(2,2)=1.d0-delta*(phi(2,2)-1.5d0*eff(2)*y(2)**2)
      phi(2,3)=phi(2,3)-delta*(2.d0*l(2)+eff(16)*y(4)+eff(15)*y(3))
      phi(2,4)=phi(2,4)-delta*(eff(16)*y(3)+eff(17)*y(4))
      phi(2,15)=-delta*l(14)
      phi(3,2)=-delta*(-eff(3)*y(3)+.5d0*eff(2)*y(2)**2)
      phi(3,3)=1.d0-delta*(-l(2)-eff(3)*y(2)-2.d0*eff(15)*y(3)-eff(16)*y(4))
      phi(3,4)=-delta*(l(3)-eff(16)*y(3))

! Energy term includes ideal gas correction.

      do i=2,15
        phi(i,1)=-delta*fprima(i)
        phi(1,i)=-be(i)*9.648529392d17+nakbt*1.5d0
      enddo

! Corrections to the energy term due to coulombian energy.
! Always active because it's more coherent due to the fact that 
! these corrections are included in the calculation of Cv.

      ne=ro*nao2
      ae=(3.d0/(4.d0*pi*ne))**third
      gam=e2/(kbt*ae)
      do i=2,15
        gamma=gam*z(i)**(5.d0/3.d0)
        if(gamma.gt.1.d0) then
           funcion=ggt1(gamma)
        else if(gamma.le.1.d0) then
           funcion=glt1(gamma)
        endif
        phi(1,i)=phi(1,i)+nakbt*funcion
      enddo
      phi(1,1)=cv

      RETURN
      END SUBROUTINE phicalc

!*******************************************************************

      FUNCTION ggt1(gamma)
      IMPLICIT NONE
      DOUBLE PRECISION,INTENT(in)::gamma
      DOUBLE PRECISION a1,b1,c1,d1,sqroot2gamma,ggt1      
      a1=-.898004d0
      b1=.96786d0
      c1=.220703d0
      d1=-.86097d0
      sqroot2gamma=sqrt(sqrt(gamma))
      ggt1=a1*gamma+b1*sqroot2gamma+c1/sqroot2gamma+d1
      return
      end function ggt1

!*******************************************************************

      FUNCTION glt1(gamma)
      implicit none
      DOUBLE PRECISION,INTENT(in)::gamma
      DOUBLE PRECISION a1,b1,c1,glt1
      a1=-.5d0*dsqrt(3.d0)
      b1=.29561d0
      c1=1.9885d0
      glt1=a1*gamma*sqrt(gamma)+b1*gamma**c1
      return
      end function glt1

!*******************************************************************

      SUBROUTINE effect(temp)
        
      USE nuclear_module

      IMPLICIT NONE

      DOUBLE PRECISION,INTENT(in)::temp

      DOUBLE PRECISION t9,t9r,t92,t93,t95,t912,t913,t923,t932,t943,t953
      DOUBLE PRECISION t9m1,t9m13,t9m23,t9m32,t9rm1,t9r32,theta
      DOUBLE PRECISION t9a,t9a2,t9a13,t9a23,t9a56
      DOUBLE PRECISION r2abe,rbeac,r3a,rev,rg3a,r24,rc28,r1216,r32
      DOUBLE PRECISION rcag,roga,roag,rnega

      t9r=temp*1.0d-09
      t9=min(10.d00,t9r)
      t92=t9*t9
      t93=t92*t9
      t95=t92*t93
      t912=dsqrt(t9)
      t913=t9**(third)
      t923=t913*t913
      t932=t9*t912
      t943=t9*t913
      t953=t9*t923
      t9m1=1.d0/t9
      t9m13=1.d0/t913
      t9m23=1.d0/t923
      t9m32=1.d0/t932
      t9rm1=1.d0/t9r
      t9r32=t9*dsqrt(t9r)
      theta=.1d0
      do i=1,16
        eff(i)=0.d0
      enddo
      do i=1,13
        l(i)=0.d0
      enddo
! **
! ** triple alpha 
! **

      r2abe=(7.40d+05*t9m32)*dexp(-1.0663d0*t9m1)&
           &+4.164d+09*t9m23*dexp(-13.49d0*t9m13-t92/0.009604d0)&
           &*(1.+0.031d0*t913+8.009d0*t923+1.732d0*t9+49.883d0&
           &*t943+27.426d0*t953)
      rbeac=(130.d0*t9m32)*dexp(-3.3364d0*t9m1)&
           &+2.510d+07*t9m23*dexp(-23.57d0*t9m13-t92/0.055225d0)&
           &*(1.d0+0.018d0*t913+5.249d0*t923+0.650d0*t9+19.176d0&
           &*t943+6.034d0*t953)
      if(t9.gt.0.08d0) then
         r3a=2.90d-16*(r2abe*rbeac)&
              &+0.1d0*1.35d-07*t9m32*dexp(-24.811d0*t9m1)
      else
         r3a=2.90d-16*(r2abe*rbeac)&
              &*(0.01d0+0.2d0*(1.d0+4.d0*dexp(-(0.025d0*t9m1)**3.263d0))&
              &/(1.d0+4.d0*dexp(-(t9/0.025d0)**9.227d0)))&
              &+0.1d0*1.35d-07*t9m32*dexp(-24.811d0*t9m1)
      endif
      eff(2)=r3a
      rev=2.d20*dexp(-84.419412d0*t9m1)

! ** 12c photodesintegration

      rg3a=rev*(t93)*r3a
      l(2)=rg3a


! **
! ** 12c+12c 
! **

      t9a=t9/(1.d0+0.0396d0*t9)
      t9a13=t9a**third
      t9a56=t9a**fsix
      r24=4.27d+26*t9a56*t9m32*dexp(-84.165d0/t9a13-2.12d-03*t93)
      eff(15)=r24

! **
! ** c12+o16 
! **

      if(t9.ge..5d0) then
            t9a=t9/(1.d0+0.055d0*t9)
            t9a2=t9a*t9a
            t9a13=t9a**third
            t9a23=t9a13*t9a13
            t9a56=t9a**fsix
            rc28=1.72d+31*t9a56*t9m32*dexp(-106.594d0/t9a13)&
                 &/(dexp(-0.18d0*t9a2)+1.06d-03*dexp(2.562d0*t9a23))
            r1216=rc28
      else
            r1216=0.d0
      endif
      eff(16)=r1216


! **
! ** 16o+16o 
! **

      r32=7.10d+36*t9m23*dexp(-135.93d0*t9m13-0.629d0*t923-0.445d0*t943&
           &+0.0103d0*t92)
      eff(17)=r32

! **
! ** 12c(a,g)16o
! **

      rcag=(1.04d+08/(t92*(1.d0+0.0489d0*t9m23)**2)&
           &*dexp(-32.120d0*t9m13-t92/12.222016d0)&
           &+1.76d+08/(t92*(1.d0+0.2654d0*t9m23)**2)*dexp(-32.120d0*t9m13)&
           &+1.25d+03*t9m32*dexp(-27.499d0*t9m1)+1.43d-02*t95&
           &*dexp(-15.541d0*t9m1))
      eff(3)=rcag

! ** 16o photodesintegration
      roga=rcag*5.13d+10*t9r32*dexp(-83.108047d0*t9rm1)
      l(3)=roga


! **
! ** 16o(a,g)20ne
! **
      roag=(9.37d+09*t9m23*dexp(-39.757d0*t9m13-t92/2.515396d0)&
           &+62.1d0*t9m32*dexp(-10.297d0*t9m1)+538.d0*t9m32&
           &*dexp(-12.226d0*t9m1)+13.d0*t92*dexp(-20.093d0*t9m1))
      eff(4)=roag

! ** 20Ne photodesintegration
      rnega=roag*5.65d+10*t9r32*dexp(-54.93807d0*t9rm1)
      l(4)=rnega
      RETURN
      END SUBROUTINE effect

! *******************************************************************
! *** Analytical calculation of  deff/dtemp

      SUBROUTINE deffcalc(temp)
        
      USE nuclear_module

      IMPLICIT NONE

      DOUBLE PRECISION,INTENT(in)::temp

      DOUBLE PRECISION t9,t92,t93,t94,t95,t912,t913,t923,t932,t943,t952,t953
      DOUBLE PRECISION t9i,t9i2,t9i3,t9i12,t9i13,t9i23,t9i32,t9i43,t9i52,t9i53
      DOUBLE PRECISION vA,vB,vC,vD,vE,vF,vG,val,r2abe,rbeac,vA56
      DOUBLE PRECISION dvA,dvB,dvC,dvD,dvE,dvF,dvG,dval,dr2abe,drbeac

      t9=temp*1.d-9
      t92=t9*t9
      t93=t92*t9
      t94=t93*t9
      t95=t94*t9
      t912=dsqrt(t9)
      t913=t9**(1.d0/3.d0)
      t923=t913*t913
      t932=t9*t912
      t943=t9*t913
      t952=t9*t932
      t953=t9*t923
      t9i=1.d0/t9
      t9i2=t9i*t9i
      t9i3=t9i2*t9i
      t9i12=1.d0/t912
      t9i13=1.d0/t913
      t9i23=1.d0/t923
      t9i32=1.d0/t932
      t9i43=1.d0/t943
      t9i52=1.d0/t952
      t9i53=1.d0/t953

! *** dl(1) and deff(2). Triple Alpha

      vA=-24.811d0*t9i
      vB=-1.0663d0*t9i
      vC=-13.49d0*t9i13-t92*104.123282d0
      vD=1.d0+.031d0*t913+8.009d0*t923+1.732d0*t9+49.883d0*t943+27.426d0*t953
      vE=-3.3364d0*t9i
      vF=-23.57d0*t9i13-18.10774106d0*t92
      vG=1.d0+.018d0*t913+5.249d0*t923+.650d0*t9+19.176d0*t943+6.034d0*t953
      dvA=24.811d0*t9i2
      dvB=1.0663d0*t9i2
      dvC=t9i43*13.49d0/3.d0-208.246564d0*t9
      dvD=(t9i23*.031d0+t9i13*16.018d0+5.196d0+t913*199.532d0+t923*137.13d0)&
           &/3.d0
      dvE=3.3364d0*t9i2
      dvF=t9i43*23.57d0/3.d0-36.21548212d0*t9
      dvG=(t9i23*.018d0+t9i13*10.498d0+1.950d0+t913*76.704d0+t923*30.17d0)/3.d0

      dl(1)=0.d0
      r2abe=7.4d5*t9i32*dexp(vB)+4.164d9*t9i23*vD*dexp(vC)
      rbeac=130.d0*t9i32*dexp(vE)+2.510d7*t9i23*vG*dexp(vF)
      dr2abe=dexp(vB)*(-1.11d6*t9i52+7.4d5*t9i32*dvB)+&
           &4.164d9*dexp(vC)*(-t9i53*vD*2.d0/3.d0+t9i23*dvC*vD+t9i23*dvD)
      drbeac=dexp(vE)*(-195.d0*t9i52+130.*t9i32*dvE)+&
           &2.510d7*dexp(vF)*(-2.d0*t9i53*vG/3.d0+t9i23*dvF*vG+t9i23*dvG)
      deff(2)=2.90d-16*(dr2abe*rbeac+r2abe*drbeac)+1.35d-8*dexp(vA)*&
           &(-1.5d0*t9i52+t9i32*dvA)


! *** dl(2) and deff(3). 12C

      vA=-84.419412d0*t9i
      vB=(1.d0+.0489d0*t9i23)**2
      vC=-32.120d0*t9i13-(t9/3.496d0)**2
      vD=(1.d0+.2654d0*t9i23)**2
      vE=-32.120d0*t9i13
      vF=-27.499d0*t9i
      vG=-15.541d0*t9i
      dvA=84.419412d0*t9i2
      dvB=-(2.d0+.0978d0*t9i23)*(.0326d0*t9i53)
      dvC=32.120d0*t9i43/3.d0-2.d0*t9/(3.496d0)**2
      dvD=-(2.d0+.5308d0*t9i23)*(.5308d0*t9i53/3.d0)
      dvE=32.120d0*t9i43/3.d0
      dvF=27.499d0*t9i2
      dvG=15.541d0*t9i2

      dl(2)=2.00d20*dexp(vA)*t93*(dvA*eff(2)+3.d0*t9i*eff(2)+deff(2))
      deff(3)=1.04d8*dexp(vC)*t9i2*(-2.d0*t9i+dvC-dvB/vB)/vB+&
           &1.76d8*dexp(vE)*t9i2*(-2.d0*t9i+dvE-dvD/vD)/vD+&
           &1.25d3*dexp(vF)*(-1.5d0*t9i52+dvF*t9i32)+&
           &1.43d-2*dexp(vG)*(5.d0*t94+dvG*t95)

! *** dl(3) and deff(4). 16O

      vA=-83.108047d0*t9i
      vB=-39.757d0*t9i13-(t9/1.586d0)**2
      vC=-10.297d0*t9i
      vD=-12.226d0*t9i
      vE=-20.093d0*t9i
      dvA=83.108047d0*t9i2
      dvB=39.757d0*t9i43/3.d0-2.d0*t9/(1.586d0)**2
      dvC=10.297d0*t9i2
      dvD=12.226d0*t9i2
      dvE=20.093d0*t9i2

      dl(3)=5.13d10*dexp(vA)*(deff(3)*t932+eff(3)*1.5d0*t912+eff(3)*t932*dvA)
      deff(4)=9.37d9*dexp(vB)*(-t9i53*2.d0/3.d0+t9i23*dvB)+&
           &62.1d0*dexp(vC)*(-1.5d0*t9i52+t9i32*dvC)+&
           &538.d0*dexp(vD)*(-1.5d0*t9i52+t9i32*dvD)+&
           &13.d0*dexp(vE)*(2.d0*t9+t92*dvE)

! *** dl(4). 20Ne

      vA=-54.903255d0*t9i
      dvA=54.903255d0*t9i2
      dl(4)=5.65d10*dexp(vA)*(deff(4)*t932+1.5d0*eff(4)*t912+eff(4)*t932*dvA)

! *** deff(15) 12C+12C

      vA=t9/(1.d0+.0396d0*t9)
      vA56=vA**(5.d0/6.d0)
      vB=-84.165d0*vA**(-1.d0/3.d0)-2.12d-3*t93

      dvA=vA*vA*t9i2
      dvB=28.055d0*dvA*vA**(-4.d0/3.d0)-6.36d-3*t92

      deff(15)=4.27d26*t9i32*dexp(vB)*(vA**(-1.d0/6.d0)*dvA*5.d0/6.d0&
           &-1.5d0*vA56*t9i+vA56*dvB)

! *** deff(16) 12C+16O

      if(t9.ge..5d0) then
        vA=t9/(1.d0+.055d0*t9)
        vA56=vA**(5.d0/6.d0)
        vB=-106.594d0*vA**(-1.d0/3.d0)
        vC=-.18d0*vA*vA
        vD=2.562d0*vA**(2.d0/3.d0)
        val=dexp(vC)+1.06d-3*dexp(vD)

        dvA=vA*vA*t9i2
        dvB=106.594d0*vA**(-4.d0/3.d0)*dvA/3.d0
        dvC=-.36d0*vA*dvA
        dvD=1.708d0*dvA*vA**(-1.d0/3.d0)
        dval=dvC*dexp(vC)+1.06d-3*dvD*dexp(vD)


        deff(16)=1.72d31*t9i32*(vA**(-1.d0/6.d0)*dvA*5.d0/6.d0&
             &-1.5d0*vA56*t9i+vA56*(dvB-dval/val))*dexp(vB)/val
      else
        deff(16)=0.d0
      endif

! *** deff(17) 16O+16O

      vA=-135.93d0*t9i13-.629d0*t923-.445d0*t943+.0103d0*t92

      dvA=45.31d0*t9i43-.629d0*t9i13*2.d0/3.d0-.445d0*t913*4.d0/3.d0+.0206d0*t9

      deff(17)=7.10d36*dexp(vA)*t9i23*(-t9i*2.d0/3.d0+dvA)

      do i=1,4
        deff(i)=deff(i)*1.d-9
      enddo
      do i=1,4
        dl(i)=dl(i)*1.d-9
      enddo

      do i=15,17
        deff(i)=deff(i)*1.d-9
      enddo
      do i=15,17
        dl(i)=dl(i)*1.d-9
      enddo

      return
      end subroutine deffcalc
! ***********************************************************************

      SUBROUTINE chempot(temp,ro)

!     CALCULATION OF THE CHEMICAL POTENTIALS
!     INPUT: Temp - Temperature; ro - Density
!     OUTPUT: mukbt & deltamukbt
        
      USE nuclear_module

      implicit none

      DOUBLE PRECISION,INTENT(in)::temp,ro

      DOUBLE PRECISION ne,kbt,ae,gam
      DOUBLE PRECISION a1,b1,c1,d1,e1
      DOUBLE PRECISION gamp,sqrootgamp,sqroot2gamp

      ne=ro*nao2
      kbt=temp*1.380658d-16
      ae=(3.d0/(4.d0*pi*ne))**(1.d0/3.d0)
      gam=e2/(kbt*ae)

      a1=-.898004d0
      b1=.96786d0
      c1=.220703d0
      d1=-.86097d0
      e1=2.520058332d0
      
      do i=1,15
         mukbt(i)=0.d0
         deltamukbt(i)=0.d0
      enddo
      deltamukbt(16)=0.d0
      deltamukbt(17)=0.d0
      
      do 2 i=2,15
         gamp=gam*z(i)**(5.d0/3.d0)
         sqrootgamp=sqrt(gamp)
         sqroot2gamp=sqrt(sqrootgamp)
         if(gamp.le.1.d0) then
            mukbt(i)=-(1.d0/dsqrt(3.d0))*gamp*sqrootgamp+(gamp**1.9885d0)&
                 &*.29561d0/1.9885d0
         else
            mukbt(i)=a1*gamp+4.d0*(b1*sqroot2gamp-c1/sqroot2gamp)&
                 &+d1*dlog(gamp)-e1
         endif
 2    enddo

      do 1 i=2,14
         deltamukbt(i)=mukbt(i)+mukbt(2)-mukbt(i+1)
 1    enddo

! Triple alpha correction.
      deltamukbt(2)=deltamukbt(2)+mukbt(2)

! Aditional deltamukbt bacuse of both extra channels of heavy ion reactions. 
! 12C+12C=24Mg+G y 16O+16O=32S+G
      deltamukbt(15)=2.d0*mukbt(3)-mukbt(5)
      deltamukbt(16)=mukbt(3)+mukbt(4)-mukbt(6)
      deltamukbt(17)=2.d0*mukbt(4)-mukbt(7)

      return
      end subroutine chempot

! *******************************************************************

      subroutine eigen(nisp,c,x)

        use nuclear_module

        implicit none
!-----------------------------------------------------------------------
!c    Prantzos, Arnould & Arcoragi  (ApJ 315,209 - 1987)
!c    inversion of the matrix (special sparse form)
!-----------------------------------------------------------------------
!c    solves the equation: phi2*x=c
!c    warning: for computation convenience the array phi2 that is fed through
!c             common/mat/ allocates an additional column, which is equal
!c             to vector c
!-----------------------------------------------------------------------
!c nisp == dimension of the true matrix a (nisp x nisp), and number of
!c         columns of array a in common/mat/. INPUT
!c c == vector containing the independent terms in the equation system. INPUT
!c x == vector containing the solution. OUTPUT
!c
!c phi2 == array of dimensions (nisp x nisp+1) containing matrix a and vector c.
!c         INPUT through common/mat/
!c
!c nds == width of the upper diagonal. INPUT through common/ceign/
!c ndi == width of the lower diagonal. INPUT through common/ceign/
!c ijd == number of columns in the left box + 1. INPUT through common/ceign/
!-----------------------------------------------------------------------
      
        DOUBLE PRECISION,dimension(15)::c,x
        integer,intent(in)::nisp

        integer nn,idel,jdel,jbal,jmax,imax,jmin
        DOUBLE PRECISION divj,som
        DOUBLE PRECISION,dimension(15)::div(15)
            
        nn=nisp
        idel=ijd-1
        jdel=ijd-1
!
!---beginning of the gaussian elimination procedure
!-(1)-elimination of the lower diagonals
!
        do 1000 jbal=ijd,nn-1
           div(jbal)=-1./phi(jbal,jbal)
           jmax=jbal+ndi
           if(jmax.gt.nn)jmax=nn
           imax=jbal+nds
           if(imax.gt.nn)imax=nn
!*vdir ignore recrdeps
           do 1001 j=jbal+1,jmax
              divj=div(jbal)*phi(j,jbal)
              do 10 i=1,ijd-1
                 if(divj.eq.0.)goto 10
                 phi(j,i)=divj*phi(jbal,i)+phi(j,i)
10            enddo
              do 20 i=jbal+1,imax
                 if(divj.eq.0.)goto 20
                 phi(j,i)=divj*phi(jbal,i)+phi(j,i)
20            enddo
1001       enddo
!*vdir ignore recrdeps
           do 1002 j=jbal+1,jmax
              if(phi(j,jbal).eq.0.)goto 1002
              c(j)=div(jbal)*phi(j,jbal)*c(jbal)+c(j)
1002       enddo
1000    enddo
        div(nn)=-1./phi(nn,nn)
!
!-(2)-elimination of the upper diagonals and of the horizontal band
!
        do 2000 jbal=nn,ijd+1,-1
           jmin=jbal-ndi
           if(jmin.lt.ijd)jmin=ijd
           do 200 j=jmin,jbal-1
              if(phi(j,jbal).eq.0.)goto 200
              divj=div(jbal)*phi(j,jbal)
              do 30 i=1,idel
                 phi(j,i)=divj*phi(jbal,i)+phi(j,i)
30            enddo
              c(j)=divj*c(jbal)+c(j)
200        enddo
           do 300 j=1,jdel
              if(phi(j,jbal).eq.0.)goto 300
              divj=div(jbal)*phi(j,jbal)
              do 40 i=1,idel
                 phi(j,i)=divj*phi(jbal,i)+phi(j,i)
40            enddo
              c(j)=divj*c(jbal)+c(j)
300        enddo
2000    enddo
        do 400 j=1,jdel
           if(phi(j,ijd).eq.0.)goto 400
           divj=div(ijd)*phi(j,ijd)
           do 50 i=1,idel
              phi(j,i)=divj*phi(ijd,i)+phi(j,i)
50         enddo
           c(j)=divj*c(ijd)+c(j)
400     enddo
!
!-(3)-gaussian elimination of the upper left square matrix
!
        do 3000 jbal=1,ijd-2
           div(jbal)=-1./phi(jbal,jbal)
           do 3001 j=jbal+1,ijd-1
              if(phi(j,jbal).eq.0.)goto 3000
              divj=div(jbal)*phi(j,jbal)
              do 60 i=jbal+1,ijd-1
                 phi(j,i)=divj*phi(jbal,i)+phi(j,i)
60            enddo
              c(j)=divj*c(jbal)+c(j)
3001       enddo
3000    enddo
        div(ijd-1)=-1./phi(ijd-1,ijd-1)
        x(ijd-1)=-div(ijd-1)*c(ijd-1)
        do 4000 jbal=ijd-2,1,-1
           som=0.
           do 70 i=ijd-1,jbal+1,-1
              som=som+phi(jbal,i)*x(i)
70         enddo
           x(jbal)=div(jbal)*(som-c(jbal))
4000    enddo
        som=0.
        do 80 i=1,idel
           som=som+phi(nn,i)*x(i)
80      enddo
        x(nn)=div(nn)*(som-c(nn))
        do 5000 jbal=nn-1,ijd,-1
           som=0.
           do 90 i=1,idel
              som=som+phi(jbal,i)*x(i)
90         enddo
           x(jbal)=div(jbal)*(som-c(jbal))
5000    enddo
        return
      end subroutine eigen

! **************************************************************
