!**********************************************************************
! Program that calls the nuclear network net14.
! See R.M. Cabezon et al. ApJ 151 (2004) for further information.
!
! Changing the initial conditions, different nuclear processes can be
! studied. (Ex. 12C+16O combustion, 4He combustion, 56Ni photodesintegration)
! If adiab is .false. the process is isochoric. 
! If adiab is .true. an adiabatic expansion follows after NSE is reached.
! If isotherm is .true. temperature is constant during combustion.
! If isotherm is .false. temperature is updated with the network.
! 
! The EOS includes relativistic electrons (Nadyozhin 1974), 
! ions (Bravo & Garcia-Senz 1992) and radiation.
!
! Screening corrections are set on in the nuclear network.
!
! Nuclear species included in the network and its corresponding index:
! 1-4He,  2-12C,   3-16O,   4-20Ne,  5-24Mg,  6-28Si,  7-32S, 
! 8-36Ar, 9-40Ca, 10-44Ti, 11-48Cr, 12-52Fe, 13-56Ni, 14-60Zn.
!
! Reactions included: 
! - Triple alpha
! - Alpha captures and photodesintegrations 12C(a,g)16O, 16O(a,g)20Ne, ...
! - Heavy Ion 12C(12C,a)20Ne, 12C(16O,a)24Mg, 16O(16O,a)28Si
! 
! Ruben M. Cabezon - Basel (2013)
!**********************************************************************


  PROGRAM testnuclear
  
    IMPLICIT NONE


!*********  Input parameters. Defined in parameters.in *********

!   x(i)               !Initial composition (x14 species)
!   temp               !Initial temperature
!   dens               !Initial density
!   nsteps             !Number of iterations
!   adiab              !Adiabatic expansion                                                                           
!   startexpansion     !Exp. starts at this step   
!   isotherm           !Isothermal test                                                                              
!   theta = 0.d0       !(<1) Select integration method                                                                 
                       !1.d0  - Explicit                                                                               
                       !0.5d0 - Semi-implicit                                                                          
                       !0.d0  - Implicit  

!**********************************************                                                                                                      
    INTEGER, PARAMETER:: niso = 14                   !Number of isotopes
    
    DOUBLE PRECISION,PARAMETER:: initial_dt = 1.d-16 !Initial timestep
    DOUBLE PRECISION,PARAMETER:: gammam1 = 5.d0/3.d0-1.d0
    
    INTEGER nsteps,startexpansion
    DOUBLE PRECISION theta
    LOGICAL adiab,isotherm,screen

    INTEGER i,j
    DOUBLE PRECISION temp,tempout,dens,delta,cv,v1,v2,sumtot,densold,deltadens,aux,adiabconst
    DOUBLE PRECISION time,deltatemp,densini,t_hd,deltatime,t0,deltad,deltat,dold,nucenergy
    DOUBLE PRECISION xmui,be,dbet,dber
    DOUBLE PRECISION,DIMENSION(niso):: x,xout
    CHARACTER*13 admy

!************ Read parameters ************
    open(1,file='parameters.in')

    do i=1,niso
       read(1,*) admy,x(i)
    enddo
    read(1,*) admy,temp
    read(1,*) admy,dens
    read(1,*) admy,nsteps
    read(1,*) admy,screen
    read(1,*) admy,adiab
    read(1,*) admy,startexpansion
    read(1,*) admy,isotherm
    read(1,*) admy,theta

!************ Checking parameters ************

    if(adiab) then
       if(isotherm) then
          print *, 'adiab and isotherm cannot be true at the same time.'
          stop
       endif
       if(theta.eq.1) then
          print *,'Theta should be 0 <= theta < 1'
          stop
       endif
       if(sum(x).ne.1.d0) then
          print *,'Wrong total composition. Sum(X)!=1'
          stop
       endif
    endif

!**********************************************

    densini=dens                      !Initial density
    t_hd=446.d0/sqrt(densini)         !Hydrodynamic timescale
    delta=initial_dt                  !Timestep
    time=delta                        !Physical time
    aux=1.d0-theta                    !theta rescaling

    open(1,file='result.d')
1   format(30(1x,es17.10))

    densold=dens
    print '(2(1x,a12),5(1x,a7),1x,a9)','Temp(K)','Dens(g/cm3)','X(4He)','X(12C)','X(16O)','X(28Si)','X(56Ni)'
    do i=1,nsteps
       if(mod(i,10).eq.0) then
          print '(2(1x,es12.5),5(1x,f7.4))', temp,dens,x(1),x(2),x(3),x(6),x(13)
       endif
       deltadens=dens-densold

!Call equation of state (electrons+ions+radiation)
       call eos(dens,temp,x,cv,v1,v2)
       v1=v1*deltadens
       v2=v2*deltadens

!Call nuclear network
       call net14(temp,tempout,delta,x,xout,dens,screen,cv,v1,v2,sumtot,nucenergy,niso,aux)

!Write output
       write(1,1) time,tempout,delta,sumtot,(xout(j),j=1,niso),dens,nucenergy

!Timestep control
       deltat=1.d60
       deltad=1.d60
       deltatemp=temp-tempout
       dold=delta
       if(deltatemp.ne.0.d0) then
          deltat=.004d0*temp*delta/abs(deltatemp)
       endif
       if(deltadens.ne.0.d0) then
          deltad=0.01d0*dens*delta/abs(deltadens)
       endif
       delta=min(deltat,deltad)
       if(delta.gt.1.5d0*dold)delta=1.5d0*dold
       delta=min(delta,.1d0)
       if(i.eq.startexpansion+1.and.adiab)delta=initial_dt

!Update quantities
       x=xout
       if(.not.isotherm)temp=tempout
       time=time+delta

       densold=dens

!******Change density here (i.e. adiabatic expansion)
       if(adiab.and.i.ge.startexpansion) then
          if(i.eq.startexpansion) then
             t0=time
             adiabconst=temp/dens**gammam1
          endif
          deltatime=time-t0
          dens=densini*exp(-deltatime/t_hd)
          temp=adiabconst*dens**gammam1
          if(temp.lt.4.d6)stop
       endif

    enddo
    
    close(1)
    
  end PROGRAM testnuclear


!**********************************************

  SUBROUTINE eos(dens,temp,x,cv,v1,v2)

    USE nuclear_module, only: niso,a,z

    IMPLICIT NONE

    INTEGER i
    LOGICAL error

    DOUBLE PRECISION, intent(in):: dens,temp
    DOUBLE PRECISION, DIMENSION(niso),intent(in):: x

    DOUBLE PRECISION, intent(out):: cv,v1,v2
    DOUBLE PRECISION, DIMENSION(niso)::anum,znum

    DOUBLE PRECISION mue,pe,dpdte,dpdre,ue,dudte,dudre
    DOUBLE PRECISION mui,ye,pii,dpdti,dpdri,ui,dudti,dudri
    DOUBLE PRECISION pr,dpdtr,dpdrr,ur,dudtr,dudrr
    DOUBLE PRECISION dens2,dpdt

    mue=0.d0
    mui=0.d0
    do i=1,niso
       mue=mue+x(i)*z(i+1)/a(i+1)
       mui=mui+x(i)/a(i+1)
    enddo
    ye=mue
    mue=1.d0/mue
    mui=1.d0/mui
    
    call eose(dens,temp,mue,pe,dpdte,dpdre,ue,dudte,dudre,error) 
!    if(error) then
!       write(*,*) 'Error en eose'
!       write(*,*) dens
!       error=.false.
!    endif
    do i=1,niso
       anum(i)=a(i+1)
       znum(i)=z(i+1)
    enddo
    call eosi(dens,temp,mui,ye,pii,dpdti,dpdri,ui,dudti,dudri,x,anum,znum,niso)
    call eosr(dens,temp,pr,dpdtr,dpdrr,ur,dudtr,dudrr)
    
    dens2=dens*dens
    dpdt=dpdte+dpdti+dpdtr
    v1=dpdt/dens2
    v2=temp*v1
    cv=dudte+dudti+dudtr

  END SUBROUTINE eos

!**********************************************

